package com.liquidnet.common.third.zxlnft.util;

import com.liquidnet.common.third.zxlnft.biz.ZxlnftBiz;
import com.liquidnet.common.third.zxlnft.config.ZxlnftConfig;
import com.liquidnet.common.third.zxlnft.dto.Nft034PublishReqDto;
import com.liquidnet.common.third.zxlnft.dto.Nft034PublishRespDto;
import com.liquidnet.common.third.zxlnft.dto.ZxlnftResponseDto;
import com.liquidnet.common.third.zxlnft.dto.nft.*;
import com.liquidnet.common.third.zxlnft.service.ZxlnftSdkService;
import com.liquidnet.commons.lang.util.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author AnJiabin <anjiabin@zhengzai.tv>
 * @version V1.0
 * @Description: TODO
 * @class: ZxlnftSdkUtil
 * @Package com.liquidnet.common.third.zxlnft.util
 * @Copyright: LightNet @ Copyright (c) 2022
 * @date 2022/2/24 16:08
 */
@Slf4j
@Component
public class ZxlnftSdkUtil {
    @Autowired
    private ZxlnftConfig zxlnftConfig;

    @Autowired
    private ZxlnftSdkService zxlnftSdkService;

    @Autowired
    private ZxlnftBiz zxlnftBiz;

    public Nft016IdentityBindQueryResp nft016IdentityBindQuery(Nft016IdentityBindQueryReq req){
        req.setAddressList(zxlnftConfig.getNftPlatformAddress());
        Nft016IdentityBindQueryResp resp = zxlnftSdkService.nft016IdentityBindQuery(req);
        return resp;
    }

    public Nft021UploadUrlResp nft021UploadUrl(Nft021UploadUrlReq req){
        Nft021UploadUrlResp resp = zxlnftSdkService.nft021UploadUrl(req);
        return resp;
    }

    public Nft022UploadSecretResp nft022UploadSecret(Nft022UploadSecretReq req){
        Nft022UploadSecretResp resp = zxlnftSdkService.nft022UploadSecret(req);
        return resp;
    }

    public ZxlnftResponseDto<Nft034PublishRespDto> nft034Publish(Nft034PublishReqDto reqDto){
        Nft034PublishReq req = Nft034PublishReq.getNew();
        BeanUtil.copy(reqDto,req);
        req.setPubKey(zxlnftConfig.getNftPlatformPubKey());
        req.setPlatformPubKey(zxlnftConfig.getNftPlatformPubKey());
        req.setHash(zxlnftBiz.getHashString(req.getUrl()));

        /**
         * 发行人的私钥签名，签名对象是(platformPubKey_pubKey_接口名_author_name_url_displayUrl_hash_desc_flag_publishCount_seriesId_seriesBeginIndex
         * _sellStatus_sellCount_metaData_operateId)
         * 接口名：publish_nft
         */
        String signMetaData = req.getPlatformPubKey().concat("_").concat("publish_nft")
                .concat("_").concat(req.getAuthor())
                .concat("_").concat(req.getName())
                .concat("_").concat(req.getUrl())
                .concat("_").concat(req.getDisplayUrl())
                .concat("_").concat(req.getHash())
                .concat("_").concat(req.getDesc())
                .concat("_").concat(req.getFlag()==null?"":req.getFlag())
                .concat("_").concat(req.getPublishCount().toString())
                .concat("_").concat(req.getSeriesId()==null?"":req.getSeriesId())
                .concat("_").concat(req.getSeriesBeginIndex().toString())
                .concat("_").concat(req.getSellStatus().toString())
                .concat("_").concat(req.getSellCount()==null?Long.toString(0): req.getSellCount().toString())
                .concat("_").concat(req.getMetaData()==null?"":req.getMetaData())
                .concat("_").concat(req.getOperateId());
        String signature = zxlnftBiz.createSign(zxlnftConfig.getNftPlatformPriKey(),signMetaData);
        req.setSignature(signature);

        String platformSignature = zxlnftBiz.createSign(zxlnftConfig.getNftPlatformPriKey(),signMetaData);
        req.setPlatformSignature(platformSignature);
        Nft034PublishResp resp = zxlnftSdkService.nft034Publish(req);

        Nft034PublishRespDto respDto = Nft034PublishRespDto.getNew();
        BeanUtil.copy(resp,respDto);
        return ZxlnftResponseDto.success(respDto);
    }
}
